@@ -7,7 +7,7 @@ use warnings FATAL => 'all';				# Enable warnings to catch errors
 
 # Initialize our version
 use vars qw( $VERSION );
-$VERSION = '0.02';
+$VERSION = '0.03';
 
 # We need Net::SSLeay or all's a failure!
 BEGIN {
@@ -27,30 +27,31 @@ BEGIN {
 			die 'Please upgrade Net::SSLeay to 1.17+ or Net::SSLeay::Handle to 0.61+';
 		} else {
 			# Finally, load our subclass :)
-			require POE::Component::SSLify::Handle;
+			require POE::Component::SSLify::ClientHandle;
+			require POE::Component::SSLify::ServerHandle;
 		}
 	}
 }
 
 # Do the exporting magic...
 require Exporter;
-use vars qw( @ISA @EXPORT );
+use vars qw( @ISA @EXPORT_OK );
 @ISA = qw( Exporter );
-@EXPORT = qw( SSLify );
+@EXPORT_OK = qw( Client_SSLify Server_SSLify SSLify_Options SSLify_GetCTX );
 
 # Bring in some socket-related stuff
 use Symbol qw( gensym );
 use POSIX qw( F_GETFL F_SETFL O_NONBLOCK EAGAIN EWOULDBLOCK );
 
-# Okay, the main routine here!
-sub SSLify {
-	# Get the socket!
-	my $socket = shift;
+# We need the server-side stuff
+use Net::SSLeay qw( die_now die_if_ssl_error );
 
-	# Validation...
-	if ( ! defined $socket ) {
-		die "Did not get a defined socket";
-	}
+# The server-side CTX stuff
+my $ctx = undef;
+
+# Helper sub to set blocking on a handle
+sub Set_Blocking {
+	my $socket = shift;
 
 	# Net::SSLeay needs blocking for setup.
 	#
@@ -87,14 +88,84 @@ sub SSLify {
 		}
 	}
 
+	# All done!
+	return $socket;
+}
+
+# Okay, the main routine here!
+sub Client_SSLify {
+	# Get the socket!
+	my $socket = shift;
+
+	# Validation...
+	if ( ! defined $socket ) {
+		die "Did not get a defined socket";
+	}
+
+	# Set blocking on
+	$socket = Set_Blocking( $socket );
+
 	# Now, we create the new socket and bind it to our subclass of Net::SSLeay::Handle
 	my $newsock = gensym();
-	tie( *$newsock, 'POE::Component::SSLify::Handle', $socket ) or die "Unable to tie to our subclass: $!";
+	tie( *$newsock, 'POE::Component::SSLify::ClientHandle', $socket ) or die "Unable to tie to our subclass: $!";
 
 	# All done!
 	return $newsock;
 }
 
+# Okay, the main routine here!
+sub Server_SSLify {
+	# Get the socket!
+	my $socket = shift;
+
+	# Validation...
+	if ( ! defined $socket ) {
+		die "Did not get a defined socket";
+	}
+
+	# If we don't have a ctx ready, we can't do anything...
+	if ( ! defined $ctx ) {
+		die 'Please do SSLify_Options() first';
+	}
+
+	# Set blocking on
+	$socket = Set_Blocking( $socket );
+
+	# Now, we create the new socket and bind it to our subclass of Net::SSLeay::Handle
+	my $newsock = gensym();
+	tie( *$newsock, 'POE::Component::SSLify::ServerHandle', $socket, $ctx ) or die "Unable to tie to our subclass: $!";
+
+	# All done!
+	return $newsock;
+}
+
+# Sets the key + certificate
+sub SSLify_Options {
+	# Get the key + cert
+	my( $key, $cert ) = @_;
+
+	Net::SSLeay::Handle->_initialize();
+
+	$ctx = Net::SSLeay::CTX_new() or die_now( "CTX_new($ctx): $!" );
+	Net::SSLeay::CTX_set_options( $ctx, &Net::SSLeay::OP_ALL ) and die_if_ssl_error( 'ssl ctx set options' );
+
+	# Following will ask password unless private key is not encrypted
+	Net::SSLeay::CTX_use_RSAPrivateKey_file( $ctx, $key, &Net::SSLeay::FILETYPE_PEM );
+	die_if_ssl_error( 'private key' );
+
+	# Set the cert file
+	Net::SSLeay::CTX_use_certificate_file( $ctx, $cert, &Net::SSLeay::FILETYPE_PEM );
+	die_if_ssl_error( 'certificate' );
+
+	# All done!
+	return 1;
+}
+
+# Returns the server-side CTX in case somebody wants to play with it
+sub SSLify_GetCTX {
+	return $ctx;
+}
+
 # End of module
 1;
 
@@ -105,14 +176,50 @@ POE::Component::SSLify - Makes using SSL in the world of POE easy!
 
 =head1 SYNOPSIS
 
+=head2 Client-side usage
+
 	# Import the module
-	use POE::Component::SSLify;
+	use POE::Component::SSLify qw( Client_SSLify );
 
 	# Create a normal SocketFactory wheel or something
 	my $factory = POE::Wheel::SocketFactory->new( ... );
 
 	# Converts the socket into a SSL socket POE can communicate with
-	eval { $socket = SSLify( $socket ) };
+	eval { $socket = Client_SSLify( $socket ) };
+	if ( $@ ) {
+		# Unable to SSLify it...
+	}
+
+	# Now, hand it off to ReadWrite
+	my $rw = POE::Wheel::ReadWrite->new(
+		Handle	=>	$socket,
+		...
+	);
+
+	# Use it as you wish...
+
+=head2 Server-side usage
+
+	# !!! Make sure you have a public key + certificate generated via Net::SSLeay's makecert.pl
+
+	# Import the module
+	use POE::Component::SSLify qw( Server_SSLify SSLify_Options SSLify_GetCTX );
+
+	# Set the key + certificate file
+	eval { SSLify_Options( 'public-key.pem', 'public-cert.pem' ) };
+	if ( $@ ) {
+		# Unable to load key or certificate file...
+	}
+
+	# Ah, I want to set some options ( not required )
+	# my $ctx = SSLify_GetCTX();
+	# Net::SSLeay::CTX_set_options( $ctx, foo );
+
+	# Create a normal SocketFactory wheel or something
+	my $factory = POE::Wheel::SocketFactory->new( ... );
+
+	# Converts the socket into a SSL socket POE can communicate with
+	eval { $socket = Server_SSLify( $socket ) };
 	if ( $@ ) {
 		# Unable to SSLify it...
 	}
@@ -131,6 +238,11 @@ POE::Component::SSLify - Makes using SSL in the world of POE easy!
 
 =head1 CHANGES
 
+=head2 0.03
+
+	First stab at the server-side code, help me test it out!
+	Refactored SSLify() into client/server side, so update your program accordingly!
+
 =head2 0.02
 
 	Made sure the IO::Handle way was used only on MSWin32
@@ -143,12 +255,7 @@ POE::Component::SSLify - Makes using SSL in the world of POE easy!
 
 This component represents the standard way to do SSL in POE.
 
-The standard way to use this module is to do this:
 
-	use POE::Component::SSLify;
-	$sslsock = SSLify( $socket );
-
-Oh, you want to set some SSL options? Then read up Net::SSLeay's POD and figure it out :)
 
 =head1 NOTES
 
@@ -166,34 +273,37 @@ that you check for errors and not use SSL, like so:
 
 	# Make socket SSL!
 	if ( $sslavailable ) {
-		eval { $socket = SSLify( $socket ) };
+		eval { $socket = POE::Component::SSLify::Client_SSLify( $socket ) };
 		if ( $@ ) {
 			# Unable to SSLify the socket...
 		}
 	}
 
-=head2 Server SSL Sockets
-
-This is not designed to be used in server/listening SSL sockets ( Because I haven't tested that ). Various POE
-developers believe that in order to have a SSL server, you would need to set up a server certificate, among other
-things. However, if this module magically works for you, let me know!
-
-This module did not undergo "serious" testing, other than simple connections to verify that the SSL part works...
-
-For further exploration, it is recommended to read the EXAMPLES section of Net::SSLeay's POD and study how they do
-server sockets, I will work on this when I have the time, but patches are welcome :)
-
 =head1 FUNCTIONS
 
-	There's only one function: SSLify()
+	There's four functions one can use:
 
-	Accepts a socket, SSLifies it, then returns a brand-new SSL-enabled socket!
+=head2 Client_SSLify
 
-	It might not be desirable, but SSLify() will die() if it is not able to do it's job...
+	Accepts a socket, returns a brand new socket SSLified
+
+=head2 Server_SSLify
+
+	Accepts a socket, returns a brand new socket SSLified
+
+	NOTE: SSLify_Options must be set first!
+
+=head2 SSLify_Options
+
+	Accepts the location of the SSL key + certificate files and does it's job
+
+=head2 SSLify_GetCTX
+
+	Returns the server-side CTX in case you wanted to play around with it :)
 
 =head1 EXPORT
 
-	Exports one function by force: SSLify()
+	Stuffs all the 4 functions in @EXPORT_OK so you have to request them directly
 
 =head1 SEE ALSO
 
